-- fire
-- Mod for Minetest 0.4
-- Licensed under the GNU GPLv3

-- Replace a burnable node by fire node
-- Spread the fire nodes on all the birnable nodes in radius by ABM
-- Destroy the fire nodes by ABM

-- TODO:
-- lighter
-- bomb

local FIRE_SPREAD_DELAY = 7
local FIRE_DISAPPEAR_DELAY = 8

function Set (list)
    local set = {}
    for _, l in ipairs(list) do set[l] = true end
    return set
end

local burnable = Set {
    "default:tree",
    "default:wood",
    "default:jungletree",
    "default:leaves"
    }

-- Fire
minetest.register_node("fire:fire", {
    drawtype = "plantlike",
    tile_images = { "fire_fire.png" },
    inventory_image = "fire_fire.png",
    paramtype = "light",
    is_ground_content = true,
    walkable = false,
    pointable = false,
    sunlight_propagates = true,
    damage_per_second = 20,
    alpha = 150,
    light_source = 13,
    post_effect_color = {a = 0, r = 0, g = 0, b = 0}
})

-- Match
minetest.register_craftitem("fire:match" , {
    image = "fire_match.png",
    stack_max = 99,

    on_use = function(item, player, pointed_thing)
        if pointed_thing.type == "node" then
            local new_node = minetest.env:get_node(pointed_thing.under)
            if burnable[new_node.name] then
            --if new_node.material.flammability > 0 then
                print("[fire] Match used on (" .. pointed_thing.under.x .. ',' .. pointed_thing.under.y .. ',' .. pointed_thing.under.z .. ')')
                if(math.random(1, 6) ~= 1) then
                    minetest.env:remove_node(pointed_thing.under)
                    minetest.env:add_node(pointed_thing.under, {name = "fire:fire"})
                end
                return true
            end
        end
        return false
    end
})

-- Match box
minetest.register_craftitem("fire:match_box", {
    image = "fire_match_box.png",
    stack_max = 99
})

-- Crafting match
minetest.register_craft({
    output = 'craft "fire:match" 4',
    recipe = {
        {'craft "default:coal_lump"' },
        {'craft "default:stick"' },
        {'craft "default:stick"' },
    }
})

-- Crafting match box
minetest.register_craft({
    output = 'craft "fire:match_box" 1',
    recipe = {
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' },
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' },
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' }
    }
})
minetest.register_craft({
    output = 'craft "fire:match" 9',
    recipe = {
        { 'craft "fire:match_box"'}
    }
})

-- Removing fire
minetest.register_abm({
    nodenames = { "fire:fire" },
    interval = FIRE_DISAPPEAR_INTERVAL,
    chance = 4,

    action = function(pos, node, _, __)
        minetest.env:remove_node(pos)
    end
})

-- Spreading fire
minetest.register_abm({
    nodenames = { "fire:fire" },
    interval = FIRE_DISAPPEAR_INTERVAL,
    chance = 4,

    action = function(pos, node, _, __)
        for i = -1, 1 do
        for j = -1, 1 do
        for k = -1, 1 do
            if (math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                local new_pos = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                local new_node = minetest.env:get_node(new_pos)
                if burnable[new_node.name] then
                --if new_node.material.flammability > 0 then
                    minetest.env:remove_node(new_pos)
                    minetest.env:add_node(new_pos, {name = "fire:fire" })
                end
            end
        end
        end
        end
    end
})
